//
//  AliAppGroupData.h
//  AliScreenShare
//
//  Created by huanli on 2021/1/9.
//  Copyright © 2021 huanli. All rights reserved.
//

#ifndef AliAppGroupData_h
#define AliAppGroupData_h

#import <Foundation/Foundation.h>

static NSString * _Nonnull kUserDefaultIPCMode = @"KEY_USER_DEFAULT_IPC_MODE"; // 宿主和插件IPC方式

static NSString * _Nonnull kCFNOnBroadcastProfileChange = @"CFN_ON_BROADCAST_PROFILE_CHANGE"; // 屏幕共享profile发生变化通知
static NSString * _Nonnull kCFNOnBroadcastFinished = @"CFN_ON_BROADCAST_FINISHED"; // 宿主停止推屏幕流通知

static NSString * _Nonnull kUserDefaultVideoProfileFps = @"KEY_USER_DEFAULT_VIDEO_PROFILE_FPS"; // fps设置通知
static NSString * _Nonnull kUserDefaultShareMode = @"KEY_USER_DEFAULT_SHARE_MODE"; // share mode设置通知

static NSString * _Nonnull kUserDefaultFrameVideo = @"KEY_USER_DEFAULT_FRAME_VIDEO"; // 接收屏幕共享video监听的Key
static NSString * _Nonnull kUserDefaultFrameAudioApp = @"KEY_USER_DEFAULT_FRAME_AUDIO_APP"; // 接收屏幕共享audio app监听的Key
static NSString * _Nonnull kUserDefaultFrameAudioMic = @"KEY_USER_DEFAULT_FRAME_AUDIO_MIC"; // 接收屏幕共享audio mic监听的Key

// type
static NSString * _Nonnull kPropDataType = @"dataType";

// video
static NSString * _Nonnull kPropFormat = @"format";
static NSString * _Nonnull kPropWidth = @"width";
static NSString * _Nonnull kPropHeight = @"height";
static NSString * _Nonnull kPropStrideY = @"strideY";
static NSString * _Nonnull kPropStrideU = @"strideU";
static NSString * _Nonnull kPropStrideV = @"strideV";
static NSString * _Nonnull kPropDataLength = @"dataLength";
static NSString * _Nonnull kPropData = @"data";
static NSString * _Nonnull kPropRotation = @"rotation";

// audio
static NSString * _Nonnull kPropSampleRate = @"sampleRate";
static NSString * _Nonnull kPropChannels = @"channels";
static NSString * _Nonnull kPropSampleSize = @"sampleSize";

static NSString * _Nonnull kPropArrayIndex = @"arrayIndex";
static NSString * _Nonnull kPropPacketCount = @"packetCount";

typedef NS_ENUM(NSUInteger, AliAppGroupDataType) {
    AliAppGroupDataTypeVideo = 1,
    AliAppGroupDataTypeAudioApp,
    AliAppGroupDataTypeAudioMic,
};

typedef NS_ENUM(NSInteger,AliScreenShareIPCMode) {
    AliScreenShareIPCMode_Default, //defalut is socket
    AliScreenShareIPCMode_Socket, //socket
    AliScreenShareIPCMode_UserDefaults, //userdefaults
};

/**
 * @brief 屏幕共享模式 支持位运算按需组合
 */
typedef NS_ENUM(NSInteger, AliScreenShareMode){
    /** 不共享音视频流 */
    AliScreenShareNone       = 0,
    /** 共享视频流 */
    AliScreenShareVideo      = 1 << 0,
    /** 共享App音频流 */
    AliScreenShareAudioApp   = 1 << 1,
    /** 共享所有音视频流 , 不支持屏幕共享Mic音频流, 请使用SDK自带的Mic音频采集 */
    AliScreenShareAll        = AliScreenShareVideo    |
                                  AliScreenShareAudioApp ,
};

@interface AliAppGroupData : NSObject

+ (bool)isUseSocket:(AliScreenShareIPCMode)mode;

+ (NSDictionary *)packetWithSampleBuffer:(CMSampleBufferRef)sampleBuffer;


+ (NSDictionary *)packetWithSampleBuffer:(CMSampleBufferRef)sampleBuffer
                                    type:(AliAppGroupDataType)type;

@end


#endif /* AliAppGroupData_h */
