//
//  AliBroadcastpacket.h
//  Tutorial
//
//  Created by huanli on 2021/1/6.
//  Copyright © 2021 tutan. All rights reserved.
//

#ifndef AliBroadcastpacket_h
#define AliBroadcastpacket_h

#import <Foundation/Foundation.h>
#import <CoreMedia/CoreMedia.h>

/**
0                   1                   2                   3                   4                   5
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|       8       |   4   | 2 |   4   |   4   |   4   |   4   |   4   |   4   |   4   |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
|magic |version |format |width |height |strideY |strideU |strideV |dataLenght |rotation|
 
0                   1                   2                   3                   4                   5
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|    dataLenght
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
|playload
 
*/
#pragma pack(1)
typedef struct ali_broadcast_packet_t {
    //! 魔法数
    uint64_t magic;
    
    //! 版本
    uint32_t version;
    
    //! 包类型
    uint16_t dataType;
    
    //! 图像类型
    uint16_t format;
    
    //! 图像宽度
    uint32_t width;
    
    //! 图像高度
    uint32_t height;
    
    //! y的行距
    uint32_t strideY;
    
    //! u的行距
    uint32_t strideU;
    
    //! v的行距
    uint32_t strideV;
    
    //! 图像大小
    uint32_t dataLenght;
    
    //! 图像方向
    uint32_t rotation;
    
    //! 音频声道数
    uint16_t channels;
    
    //! 音频采样率
    uint16_t sampleRate;
    
    //! 音频sample数
    uint16_t sampleSize;
    
    //! pts
    uint64_t pts;
    
} ali_broadcast_packet_t;
#pragma pack()

static const int64_t kMagicNumber = 0xFF;
static const uint16_t AliBroadcastPort = 9898;
static const uint32_t AliBroadcastVersion = 1;
static const uint32_t AliBroadcastPacketHeadSize = sizeof(ali_broadcast_packet_t); // 先申请ali_broadcast_packet_t大小
typedef NS_ENUM(NSUInteger, AliBroadcastDataType) {
    AliBroadcastDataTypeVideo = 1,
    AliBroadcastDataTypeAudioApp,
    AliBroadcastDataTypeAudioMic,
};
@interface AliBroadcastPacket : NSObject

//! 报文头
@property (nonatomic, assign) ali_broadcast_packet_t header;

//! 数据
@property (nonatomic, strong) NSData *data;

//! AliBroadcastPacket转NSData对象
+ (NSData *)packetToNSData:(AliBroadcastPacket *)packet;

//! NSData初始化AliBroadcastPacket对象
+ (instancetype)packetWithNSData:(NSData *)data;

//! uint8_t *初始化AliBroadcastPacket对象
+ (instancetype)packetWithRawData:(uint8_t *)data;

//! CMSampleBufferRef转AliBroadcastPacket对象
+ (instancetype)packetWithSampleBuffer:(CMSampleBufferRef)sampleBuffer;

+ (instancetype)packetWithSampleBuffer:(CMSampleBufferRef)sampleBuffer type:(AliBroadcastDataType)type;

//! 插件版本是否兼容
- (BOOL)isMatchedVersion;

- (int)packetSize;

@end


#endif /* AliBroadcastpacket_h */
