//
//  AliScreenShareExt.h
//  Tutorial
//
//  Created by huanli on 2020/12/28.
//  Copyright © 2020 tutan. All rights reserved.
//

#ifndef AliScreenShareExt_h
#define AliScreenShareExt_h

#import <Foundation/Foundation.h>
#import <CoreMedia/CoreMedia.h>
#import <ReplayKit/ReplayKit.h>

NS_ASSUME_NONNULL_BEGIN

static const int32_t kAudioPacketPoolSize = 5;
static const int32_t kVideoProfile_Fps = 5;

@protocol AliScreenShareExtDelegate;

/// 屏幕分享主入口类
@interface AliScreenShareExt : NSObject

/// 获取单例
+ (instancetype)sharedInstance;

/// 初始化方法
///
/// 需要在 RPBroadcastSampleHandler 的实现类中的 broadcastStartedWithSetupInfo 方法中调用
/// @param appGroup App group ID
/// @param delegate 回调对象
- (void)setupWithAppGroup:(NSString * _Nonnull)appGroup delegate:(id<AliScreenShareExtDelegate>)delegate;


/// 录屏完成方法
///
/// 通过系统控制中心停止录屏时，会回调 RPBroadcastSampleHandler.broadcastFinished，在 broadcastFinished 方法中调用
- (void)finishBroadcast:(void(^_Nullable)(void))completion;

/// 音视频发送方法
///
/// 需要在 RPBroadcastSampleHandler 的实现类中的 processSampleBuffer: 方法中调用
/// @param sampleBuffer 系统回调的音视频帧
/// @param type 音视频帧类型
- (void)sendSampleBuffer:(CMSampleBufferRef)sampleBuffer
                    type:(RPSampleBufferType)type;

@end

@protocol AliScreenShareExtDelegate <NSObject>
/// 异常录屏完成回调
///
/// @param broadcast 发出回调的实例
/// @param error 结束原因NSError
- (void)finishBroadcastWithError:(AliScreenShareExt *)broadcast error:(NSError *)error;
@end


NS_ASSUME_NONNULL_END
#endif /* AliScreenShareExt_h */
